package ru.dbotthepony.mc.otm.network.android;

import net.minecraft.client.Minecraft;
import net.minecraft.network.FriendlyByteBuf;
import net.minecraft.sounds.SoundSource;
import net.minecraftforge.api.distmarker.Dist;
import net.minecraftforge.fml.DistExecutor;
import net.minecraftforge.network.NetworkEvent;
import ru.dbotthepony.mc.otm.capability.MatteryCapability;
import ru.dbotthepony.mc.otm.capability.android.AndroidCapabilityPlayer;
import ru.dbotthepony.mc.otm.registry.MSoundEvents;

import java.util.function.Supplier;

public record AndroidStatusPacket(Type type, boolean status, boolean playSound) {
	public enum Type {
		IS_ANDROID,
		WILL_BECOME_ANDROID,
	}

	public void write(FriendlyByteBuf buffer) {
		buffer.writeByte(type.ordinal());
		buffer.writeBoolean(status);
		buffer.writeBoolean(playSound);
	}

	public void play(Supplier<NetworkEvent.Context> context) {
		context.get().setPacketHandled(true);
		context.get().enqueueWork(() -> {
			DistExecutor.unsafeRunWhenOn(Dist.CLIENT, () -> this::playClient);
		});
	}

	public void playClient() {
		var ply = Minecraft.getInstance().player;

		if (ply != null) {
			ply.getCapability(MatteryCapability.ANDROID).ifPresent(cap -> {
				if (cap instanceof AndroidCapabilityPlayer pcap) {
					if (type == Type.IS_ANDROID)
						pcap.isAndroid = status;
					else if (type == Type.WILL_BECOME_ANDROID)
						pcap.willBecomeAndroid = status;
				}

				if (playSound) {
					ply.level.playSound(ply, ply, MSoundEvents.INSTANCE.getPLAYER_BECOME_ANDROID(), SoundSource.PLAYERS, 1f, 1f);
				}
			});
		}
	}

	public static AndroidStatusPacket read(FriendlyByteBuf buffer) {
		return new AndroidStatusPacket(Type.values()[buffer.readByte()], buffer.readBoolean(), buffer.readBoolean());
	}
}
